#include "Status.h"
#include "Vram.h"
#include "Stage.h"
#include "Main.h"
#include "Movable.h"
#include "Actor.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"
#include "CopyMemory.h"

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus() 
{
    PrintS(Vram + 1, "SCORE");
    PrintS(Vram + 7, "HI-SCORE");
    PrintS(Vram + 16, "STAGE");
    PrintByteNumber2(Vram + VramRowSize + 18, CurrentStage + 1);
    PrintS(Vram+ 22, "TIME");

    {
        ptr<byte> pVram;
        pVram = Vram + 27;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            if (i > 2) {
                Put2C(pVram, Char_Remain);
                pVram += 2 * VramStep;
                *pVram = Char_Space;
                ++pVram;
                *pVram = Char_Space;
                pVram += VramRowSize - VramStep;
                *pVram = i + 0x10;
                ++pVram;
                *pVram = Char_Space;
            }
            else {
                do {
                    Put2C(pVram, Char_Remain);
                    pVram += 2 * VramStep;
                    --i;
                } while (i > 0);
            }
        }
    }

    PrintScore();
    PrintTime();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + VramRowSize, Score);
    PrintC(pVram, '0');

    pVram = PrintNumber5(Vram + VramRowSize + 9, HiScore);
    PrintC(pVram, '0');
}

void PrintTime()
{
    PrintByteNumber3(Vram + VramRowSize + 23, StageTime);
}

static void EraseWindow() 
{
    constexpr byte width = 12;
    constexpr byte height = 3;    
    ptr<byte> pVram;

    pVram = Vram + VramRowSize * 11 + 10;
    repeat (height) {
        repeat (width) {
            pVram = PrintC(pVram, ' ');
        }
        pVram += VramRowSize - width;
    }
}

void PrintGameOver()
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 12 + 11, "GAME OVER");
}

void PrintTimeUp() 
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 12 + 12, "TIME UP");
}

void Title()
{
    static const byte[] TitleBytes = {
        //	A
        0x00, 0x76, 0x66, 0x73, 
        0x68, 0x72, 0x00, 0x4e, 
        0x68, 0x64, 0x6e, 0x4e, 
        0x7a, 0x78, 0x00, 0x6e, 
        //	S
        0x61, 0x64, 0x6e, 0x63, 
        0x7a, 0x63, 0x54, 0x73, 
        0x61, 0x73, 0x00, 0x4e, 
        0x00, 0x6e, 0x6e, 0x78, 
        //	C
        0x00, 0x76, 0x6e, 0x63, 
        0x68, 0x72, 0x00, 0x00, 
        0x7a, 0x63, 0x00, 0x54, 
        0x00, 0x7a, 0x6e, 0x78, 
        //	E
        0x68, 0x64, 0x6e, 0x78, 
        0x68, 0x63, 0x54, 0x73, 
        0x68, 0x72, 0x00, 0x00, 
        0x7a, 0x6e, 0x6e, 0x78, 
        //	N
        0x68, 0x63, 0x00, 0x4e, 
        0x68, 0x4e, 0x63, 0x4e, 
        0x68, 0x72, 0x66, 0x4e, 
        0x7a, 0x78, 0x00, 0x6e, 
        //	D
        0x68, 0x64, 0x66, 0x73, 
        0x68, 0x72, 0x00, 0x4e, 
        0x68, 0x72, 0x61, 0x64, 
        0x7a, 0x6e, 0x6e, 0x00, 
   };

    ClearScreen();

    HideAllSprites();
    PrintStatus();
    {
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 7 + 4;
        p = TitleBytes;
        repeat (6) {
            repeat (4) {
                repeat (4) {
                    *pVram = *p;
                    ++pVram;
                    ++p;
                }
                pVram += VramRowSize - 4;
            }
            pVram += 4 - VramRowSize * 4;
        }
    }
    PrintS(Vram + VramRowSize * 19 + 9, "PUSH \"Z\" KEY");
    PrintS(Vram + VramRowSize * 20 + 9, "OR \"X\" KEY");
    PrintS(Vram + VramRowSize * 23 + 20, "INUFUTO 2022");
}